/* -----------------------------------------------------------------------------
   A procedure that initiates the data dump for all tables, or a
   particular table. This is mostly achieved by dynamically generataing some
   include files, which are then executed by ptmd2.p
   called by:   The procedure editor or your program
   inputs:      the table to dump or the word "ALL"
   outputs:     a test file with the required data

   Copyright (c)2001 KM Newnham mark@newnhams.com
   Version 1.0 Original release 21st Dec 2001

   -----------------------------------------------------------------------------
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   -------------------------------------------------------------------------- */

   
/*************************************************************************/
/* Radical changes by Scott Auge sauge@amduus.com  2006                  */
/*                                                                       */
/* THEORY OF OPERATION:                                                  */
/*                                                                       */
/* Like before, we generate a program that will FOR EACH through the     */
/* data and generate SQL statements for Progress V7 or better.  (V7 is   */
/* useless for dynamic queries so we have to do it the hard way.  This   */
/* requires a QUERY or COMPILE license to work right.)                   */
/*                                                                       */
/* This program does it a little differently than before - we slap a     */
/* bunch of include files that have repeating tokens and the like and    */
/* include them in the generated routine                                 */
/*                                                                       */
/* The main change is this program generated 4GL code AND SQL code. It   */
/* has been changed to generate out specifically 4GL code and then run   */
/* it for generation of SQL code for MySQL.                              */
/*                                                                       */
/* Also broke up the one big-um routine into some modular goodies.       */
/* Will work the same as documented so that doesn't need to change.      */
/*************************************************************************/


/*************************************************************************/
/* Send a specific table name or "ALL" for all the tables.               */
/*************************************************************************/

def input parameter ipFile      as char format "x(30)"         NO-UNDO.

DEFINE STREAM SRC.
DEFINE STREAM INC.

&GLOBAL-DEFINE SRC PUT STREAM SRC UNFORMATTED 
&GLOBAL-DEFINE INC PUT STREAM INC UNFORMATTED


def var intCurrentExtent   as integer                      NO-UNDO.
def var logIsArray         as logical                      NO-UNDO.
def var intExtents         as integer                      NO-UNDO.
def var charFieldName      as char                         NO-UNDO.
def var charLastFieldName  as char format "x(50)"          NO-UNDO.
def var charFileName       as character format "x(50)"     NO-UNDO.
def var intFieldCount      as integer                      NO-UNDO.
def var logFirstField      as logical initial yes          NO-UNDO.
def var lEmpty		   as logical initial false	   NO-UNDO.
def var cTemp		   as char                         NO-UNDO.
def var lAll		   as logical                      no-undo.
def var cWarning           as character                    no-undo.
def var cSQLFileName       as character                    no-undo.
def var cListOfFields      as character                    no-undo.
def var cCurField          as character                    no-undo.
                           
DEF VAR mysqlcmd           AS CHAR INITIAL '~\adg~\web~\mysql~\bin~\mysql -u root' NO-UNDO.     

FUNCTION QuoteIt RETURNS CHARACTER (INPUT cString AS CHARACTER) FORWARD.

/****************************************************************************/
/* If we are doing em all, then set a flag so we know further on AND delete */
/* the mysqlld.bat file which contains multiple mysql client calls with the */
/* the given tables so we don't accidently load data we don't want to.      */
/****************************************************************************/

if (ipFile = "ALL") then do:
	
  ASSIGN lAll = true.

  RUN DelFile ("mysqlld.bat").
  
end.

/****************************************************************************/
/* Start working our tables of interest.  It is a one or all kind of thing. */
/****************************************************************************/

for each _file where (( _file-name  < '_' and ipFile  = "ALL" )
		  or  ( _file-name = ipFile    and ipFile <> "ALL"))
		 AND _hidden = FALSE no-lock:

   ASSIGN ipFile = _file._file-name.

   status default "Processing " + ipFile.
                                                                            
   /**********************************************************************/
   /* A test to determine if it is worth dumping anything out.  This way */
   /* we don't get any half INSERT statements created later on.          */
   /**********************************************************************/

   
   IF SEARCH('avail.p') <> ? THEN RUN DelFile ("avail.p").
   
   ASSIGN cTemp = "def output parameter l as log no-undo.~nfind first " + _file-name + " no-lock no-error.~nIf not available " + _file-name + " then l = true.~n".
   
   output to avail.p.
   put unformatted cTemp.         
   output close.
   
   run avail.p (output lEmpty).
   
   if lEmpty then do:
     
     ASSIGN charFileName = _file._dump-name + '.SQL'.
	  
     if search(charFileName) = ? then next.
       
     RUN DelFile (charFileName).
     
     NEXT.
     
   end. /* IF lEmpty */

    
   /**********************************************************************/
   /* Tag on multiple iterations of load statements into the batch file. */
   /* We do this to continue functionality from the original version.    */
   /**********************************************************************/
    
   if lAll then do:
     charFileName = _file._dump-name + '.SQL'.
	 if search(charFileName) <> ? then do:
		output to "mysqlld.bat" append.
   
		put unformatted mysqlcmd " < " charFileName skip.
        output close.    
     end.
   end.

   /**********************************************************************/ 
   /* Lets start writing the 4GL program that will generate the SQL.     */
   /* We don't need an SQL stream but it helps keep us straight which    */
   /* stream is in the generated program and which is actually in this   */
   /* one!                                                               */
   /**********************************************************************/ 
   
   OUTPUT STREAM SRC TO gensql.p.

   {&SRC} "/* Working table " _file._file-name " */" SKIP.
   {&SRC} "/* WARNING: Generated Code From " PROGRAM-NAME(1) " */" SKIP.   
   {&SRC} "DEFINE STREAM SQL." SKIP.
   {&SRC} "DEFINE VARIABLE cTemp AS CHARACTER NO-UNDO." SKIP.   
   
   /**********************************************************************/ 
   /* Help our 4GL program determine which SQL file to write.            */
   /**********************************************************************/ 
   
   ASSIGN cTemp = _file._dump-name + ".SQL".
   {&SRC} "OUTPUT STREAM SQL TO " cTemp "." SKIP. 
   
   /**********************************************************************/ 
   /* Determine which "database" in MYSQL to use.  We default to the     */
   /* logical name of the Progress DB.                                   */
   /**********************************************************************/    
   
   ASSIGN charFileName = LDBNAME(1).
   run d2us.p(input-output charFileName, output cWarning).
   
   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("USE " + charFileName + ";") " SKIP." SKIP.

   /**********************************************************************/  
   /* Count up fields, etc. for informational purposes.                  */
   /**********************************************************************/  

   ASSIGN    
   logFirstField = yes
   intFieldCount = 0.
   
   for each _field of _file.
	   ASSIGN intFieldCount = intFieldCount + 1.
   end.

   /**********************************************************************/
   /* Here we figure out our file name.  In the 4GL code we can use what */
   /* is there.  In the SQL code, the progress file name may be a MySQL  */
   /* keyword.  (BIN is a common one!)  So SQL tables are prefixed with  */
   /* an X_ if they are a MySQL keyword.                                 */
   /**********************************************************************/
   
   ASSIGN charFileName = _file._file-name
          cSQLFileName = charFileName.
   
   run d2us.p (input-output cSQLFileName, output cWarning).

   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("LOCK TABLE " + cSQLFileName + " WRITE;") " SKIP." SKIP.
   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("SET AUTOCOMMIT=0;") " SKIP." SKIP.

   /**************************************************************************/
   /*                              PLEASE READ                               */
   /*                                ______                                  */
   /*                             .-"      "-.                               */
   /*                            /            \                              */
   /*                _          |              |          _                  */
   /*               ( \         |,  .-.  .-.  ,|         / )                 */
   /*                > "=._     | )(__/  \__)( |     _.=" <                  */
   /*               (_/"=._"=._ |/     /\     \| _.="_.="\_)                 */
   /*                      "=._ (_     ^^     _)"_.="                        */
   /*                          "=\__|IIIIII|__/="                            */
   /*                         _.="| \IIIIII/ |"=._                           */
   /*               _     _.="_.="\          /"=._"=._     _                 */
   /*              ( \_.="_.="     `--------`     "=._"=._/ )                */                                          
   /*               > _.="                            "=._ <                 */
   /*              (_/                                    \_)                */
   /*                                                                        */
   /* WARNING: NOTE THE LINE BELOW INSERTS THE INSTRUCTION TO DELETE ALL     */
   /*          DATA IN THE DESTINATION TABLE.                                */
   /**************************************************************************/     
   
   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("EMPTY TABLE " + cSQLFileName + ";") " SKIP." SKIP.   
   
   /**********************************************************************/ 
   /* Figure out the last field's name so we know it when we find it.    */
   /* Also build up a field list so we can be sure of the order of       */
   /* values sent in the INSERT statement.                               */
   /**********************************************************************/    

   ASSIGN intCurrentExtent = 0
          cListOfFields = "PROGRESS_RECID,".
          
   for each _field of _file:
     
     ASSIGN charLastFieldName = _field-name.
   
     run d2us.p (input-output charLastFieldName, output cWarning).
     
     IF _field._extent > 0 THEN DO:
       
       ASSIGN intCurrentExtent = 1.
       REPEAT WHILE intCurrentExtent <= _field._extent:
         
         ASSIGN cListOfFields = cListOfFields 
                              + charLastFieldName + "__" + STRING(intCurrentExtent)
                              + ",".
                              
         ASSIGN intCurrentExtent = intCurrentExtent + 1.
                              
       END.
       
     END.
       
     ELSE
       ASSIGN cListOfFields = cListOfFields + charLastFieldName + ",".
     
   end. /* FOR EACH _Field */
   
   /* Take off the last comma */
   
   ASSIGN cListOfFields = SUBSTRING(cListOfFields, 1, LENGTH (cListOfFields) - 1).    

   {&SRC} "FOR EACH " trim(charFileName) " NO-LOCK: " SKIP.   
     
   {&SRC} "  PUT STREAM SQL UNFORMATTED " QuoteIt("INSERT INTO " + cSQLFileName + " (" + cListOfFields + ") VALUES (") "." SKIP.


    /* -------------------------------------------
       Add the recid to create a primary index
       ------------------------------------------- */

   {&SRC} "  PUT STREAM SQL UNFORMATTED " "RECID(" + _file-name + ") " QuoteIt(",") "." SKIP.
   
   for each _field of _file.


	intExtents = 0.
	logIsArray = false.
	if _field._extent > 0 then do:
	    intExtents = 1.
	    logIsArray = true.
	end.

	repeat while intExtents <= _field._extent:
 
            /****************************************************************/
            /* Determine if we need an opening ' or not                     */
            /****************************************************************/
        
	    if logFirstField = no then do:
		if _data-type = "character" or _data-type = "date" then do:
                  {&SRC} "  PUT STREAM SQL UNFORMATTED "  QuoteIt(",~'") "." SKIP.
		end.
		else do:
                  {&SRC} "  PUT STREAM SQL UNFORMATTED "  QuoteIt(", ") "." SKIP .
		end.
	    end.
	    else do:
		if _data-type = "character" or _data-type = "date" then do:
                  {&SRC} "  PUT STREAM SQL UNFORMATTED "  " ~"~'~"." SKIP.
		end.
		else do:
                  {&SRC} "  PUT STREAM SQL UNFORMATTED "  QuoteIt(" ") "." SKIP.
		end.
	    end.
            
            /****************************************************************/            
            /* Figure out how we refer to the field in the generated code.  */
            /****************************************************************/
            
	    if logIsArray = false then do:
		intExtents = 1.
		charFieldName = _field-name.
	    end.
	    else do:
		charFieldName = _field-name + "[" + string(intExtents) + "]".
		intExtents = intExtents + 1.
	    end.

	    /* -----------------------------------------------
	       Extract the date so that is can be built in ISO
	       format. This should allow European format dates
	       to be processed correctly as well as US
	       ----------------------------------------------- */
               
	    if _data-type = "date" then do:             

              {&SRC}  "ASSIGN cTemp = string(year(" + charFieldName + "),~"9999~") + ~"-~" + string(month(" + charFieldName + "),~"99~") + ~"-~" + string(day(" + charFieldName + "),~"99~")." SKIP.

              {&SRC} "PUT STREAM SQL UNFORMATTED " "cTemp." SKIP.
                
	    end.
            
	    else if _data-type = "character" then do:

              {&SRC} SKIP "  ASSIGN cTemp = " charFieldName "." SKIP.
              {&SRC} "  run cwrite.p(input-output cTemp)." skip.

              {&SRC} "  PUT STREAM SQL UNFORMATTED cTemp ." SKIP.
              

	    end.
	    /* ------------------------------------------
	       Convert logicals to integers because we
	       are going to put them in a tinyint field
	       ------------------------------------------ */
	    else if _data-type = "logical" then do:

              {&SRC} "  PUT STREAM SQL UNFORMATTED "  "integer(" charFieldName ") SKIP." SKIP.

               
	    end.
	 /* ------------------------------------------
	       Currently Convert version 9 RAW DATA to
	       nothing
	       ------------------------------------------ */
	    else if _data-type = "raw" then do:
              {&SRC} "  PUT STREAM SQL UNFORMATTED "   "~"NULL~"" .
	    end.

	    else if _data-type = "recid" then do:
              {&SRC} "  PUT STREAM SQL UNFORMATTED "  "if " charFieldName "= ? then 0 else integer(" charFieldName ").".
	    end.
            
	    else do:
              /* ---------------------------------------
              Convert an unknown value to zero,
              otherwise, just write the value
              this is for decimals, integers
              -------------------------------------- */   

              {&SRC} "  PUT STREAM SQL " "if " + charFieldName + " = ? then string(0) else string(" + charFieldName ")." SKIP.
              
	    end.         
                       
            /****************************************************************/
            /* Figure out on end of data if we need a ' or not.             */
            /****************************************************************/
            
            if _field._data-type = "character" or _field._data-type = "date" then do:
                                                            
              {&SRC} "  PUT STREAM SQL UNFORMATTED "  QuoteIt("'") "." SKIP.
            
            end.                   
            else do:
            
              {&SRC} "  PUT STREAM SQL UNFORMATTED ~"~"." SKIP.  
            
            end.

	    logFirstField = no.
            
	end. /* REPEAT  while intExtents <= _field._extent */
        
	logFirstField = no.

    end. /* FOR EACH _Field */

    
   {&SRC} "  PUT STREAM SQL UNFORMATTED " QuoteIt(");") "SKIP." SKIP.
   
   /**********************************************************************/
   /* Wrap up the FOR EACH our generated program produces and do house-  */
   /* keeping.                                                           */
   /**********************************************************************/
   
   {&SRC} "END. /* FOR EACH */" SKIP.
   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("UNLOCK TABLES;") " SKIP." SKIP.
   {&SRC} "PUT STREAM SQL UNFORMATTED " QuoteIt("COMMIT;") " SKIP." SKIP.

   {&SRC} "OUTPUT STREAM SQL CLOSE." SKIP.    
   
   OUTPUT STREAM SRC CLOSE.

   /**********************************************************************/
   /*  Now we have built the includes, lets run the program to           */
   /*  extract the data.  We pause 2 seconds so the client realizes the  */
   /* gensql.p has changed!  Otherwise it goes and uses the cached       */
   /* version.                                                           */
   /**********************************************************************/

   PAUSE 2 NO-MESSAGE.
   run gensql.p.

end.

/**************************** Internal Procedures ****************************/

/**********************************************************************/
/* Given a file name, delete it.  Used for Progress Versions without  */
/* OS-DELETE.                                                         */
/**********************************************************************/

PROCEDURE DelFile:

  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cTemp AS CHARACTER NO-UNDO.

  IF OPSYS = 'unix' THEN DO:
     ASSIGN cTemp = 'rm ' + cFileName.
     UNIX SILENT VALUE(cTemp).
   END.
   ELSE DO:
     ASSIGN cTemp = "del " + cFileName.
     dos silent value(cTemp).
   END.    

END. /* PROCEDURE DelFile */
    
/**********************************************************************/
/* When we are quoting quotes it can be difficult to keep the two     */
/* straight in the head, so this function can be useful.              */
/**********************************************************************/

FUNCTION QuoteIt RETURNS CHARACTER (INPUT cString AS CHARACTER):
  
  RETURN "~"" + cString + "~"".

END.

